package com.unitech;

import com.thingmagic.Reader;
import com.thingmagic.SerialTransportRS232;
import com.thingmagic.TMConstants;
import com.unitech.util.LoggerUtil;

public class ReaderConnect {
	private static final String TAG = "ReaderConnect";
	private static int BAUDRATE_TARGET = 460800; // 115200, 230400, 460800, *921600

    private static Reader reader;

	public static void setTrace(Reader r, String args[]) {
		System.out.println("setTrace " );
		if (args[0].toLowerCase().equals("on")) {
			r.addTransportListener(r.simpleTransportListener);
		}
	}
	
	public static Reader connect(String uriString, Reader.Region region, boolean isHsUART) throws Exception
	{
		try
		{
            if (reader == null) {
                Reader.setSerialTransport("ute", new SerialTransportRS232.Factory());
                reader = Reader.create(uriString);
//                setTrace(reader, new String[]{"on"});
            }
            reader.connect();

			if (((int)reader.paramGet(TMConstants.TMR_PARAM_BAUDRATE)) != BAUDRATE_TARGET) {
                LoggerUtil.debug(TAG, String.format("Set baudrate to %d", BAUDRATE_TARGET));
                reader.paramSet(TMConstants.TMR_PARAM_BAUDRATE, BAUDRATE_TARGET);
            }

			LoggerUtil.debug(TAG, "Set Region :" + region);
			reader.paramSet("/reader/region/id", region);

			LoggerUtil.debug(TAG, "Reader current region :" + (Reader.Region) reader.paramGet("/reader/region/id"));

			Reader.Region[] supportedRegions = (Reader.Region[]) reader
					.paramGet(TMConstants.TMR_PARAM_REGION_SUPPORTEDREGIONS);

			String supportRegions = "Support Regions : ";
			for (Reader.Region supportedRegion : supportedRegions) {
				if (supportedRegion != null) {
					supportRegions += supportedRegion.toString() + ",";
                }
			}
			LoggerUtil.debug(TAG, supportRegions);
		} catch (Exception ex) {
			throw ex;
		}

		return reader;
	}

	public static void setBaudrate (int baudrate) {
        BAUDRATE_TARGET = baudrate;
    }

    public static int getBaudrate () {
        return BAUDRATE_TARGET;
    }
}
