package com.unitech.rfidreader.Listener;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.os.AsyncTask;
import android.view.View;

import com.thingmagic.Reader;
import com.unitech.ReaderConnect;
import com.unitech.rfidreader.R;
import com.unitech.rfidreader.ReaderActivity;
import com.unitech.util.LoggerUtil;

import java.io.File;

@SuppressLint("StaticFieldLeak")
public class ConnectionListener implements View.OnClickListener {

    private ReaderActivity mReaderActivity;

    private ProgressDialog pDialog;

    private final String TAG = "ConnectionListener";

    private String UART;
    private boolean isHsUART = false;

    public ConnectionListener(ReaderActivity readerActivity) {
        mReaderActivity = readerActivity;
        pDialog = new ProgressDialog(readerActivity);
        pDialog.setCancelable(false);
        pDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        findAllViewsById();
    }

    private void findAllViewsById() {
    }

    @Override
    public void onClick(View arg0) {
        mReaderActivity.initUI();

        if (mReaderActivity.getConnectBtnText().equalsIgnoreCase(mReaderActivity.getString(R.string.Connect))) {
            File devFolder = new File("/dev");
            if (devFolder.exists() && devFolder.isDirectory()) {
                if (new File("/dev/ttyHS3").exists()) {
//                    UART = "tmr:///dev/ttyHS3";
                    UART = "ute:///dev/ttyHS3";
                    isHsUART = true;
                } else if (new File("/dev/ttyMSM2").exists()) {
                    UART = "tmr:///dev/ttyMSM2";
                    isHsUART = false;
                } else if (new File("/dev/ttyHS1").exists()) {
                    UART = "ute:///dev/ttyHS1";
                    isHsUART = true;
                }
            }
            System.out.println("UART :" + UART);
        }

        ReaderConnectionThread readerConnectionThread = new ReaderConnectionThread(
                UART, mReaderActivity);
        readerConnectionThread.execute();
    }

    private class ReaderConnectionThread extends AsyncTask<Void, Void, String> {
        private ReaderActivity mReaderActivity;
        private String uriString;
        private String operation;
        private boolean operationStatus = true;
        private boolean doConnect;

        ReaderConnectionThread(String requestedQuery, ReaderActivity activity) {
            this.mReaderActivity = activity;
            this.uriString = requestedQuery;
            this.operation = mReaderActivity.getConnectBtnText();

            if (operation.equalsIgnoreCase("Connect"))
                doConnect = true;
            else
                doConnect = false;
        }

        @Override
        protected void onPreExecute() {
            LoggerUtil.debug(TAG, "** ReaderConnectionThread.onPreExecute **");
            mReaderActivity.connectBtnEdit(false);
            if (doConnect) {
                pDialog.setMessage("Connecting. Please wait...");
            } else {
                mReaderActivity.readBtnEdit(false);
                mReaderActivity.tagOpBtnEdit(false);
                pDialog.setMessage("Disconnecting. Please wait...");
            }
            pDialog.show();

            mReaderActivity.initUI();
            mReaderActivity.setErrorMsg("");
            mReaderActivity.showErrorMsg(false);
//            ServiceListener.clearTagRecords();
        }

        @Override
        protected String doInBackground(Void... params) {
            String exception = "Exception :";
            try {
                if (doConnect) {
                    LoggerUtil.debug(TAG, "Start connecting");

                    LoggerUtil.debug(TAG, "uriString=" + uriString);

                    Reader.Region region = Reader.Region.valueOf(mReaderActivity.getSelectedRegionList());

                    LoggerUtil.debug(TAG, "Selected Region :" + region);

//                    mReaderActivity.RFIDModulePowerCtrl(true);

                    mReaderActivity.reader = ReaderConnect.connect(uriString, region, isHsUART);

                    LoggerUtil.debug(TAG, "Reader Connected");

                } else {
                    mReaderActivity.reader.destroy();
                    mReaderActivity.reader = null;

                    LoggerUtil.debug(TAG, "Reader Disconnected");
                }

            } catch (Exception ex) {
                operationStatus = false;
                if (ex.getMessage().contains("Connection is not created")
                        || ex.getMessage().startsWith("Failed to connect")) {
                    exception += "Failed to connect to RFID reader";
                } else {
                    exception += ex.getMessage();
                }
                LoggerUtil.error(TAG, "Exception while do " + operation, ex);
            }
            return exception;
        }

        @Override
        protected void onPostExecute(String exception) {
            pDialog.dismiss();

            if (!operationStatus) {
                AlertDialog deleteAlert = new AlertDialog.Builder(mReaderActivity).create();
                deleteAlert.setTitle(operation);
                deleteAlert.setMessage(operation + " Timeout");
                deleteAlert.show();
            }

            LoggerUtil.debug(TAG, "** ReaderConnectionThread.onPostExecute **");
            if (!operationStatus) {
                LoggerUtil.debug(TAG, "OperationStatus is false");
                if (doConnect) {
                    mReaderActivity.readBtnEdit(false);
                    mReaderActivity.tagOpBtnEdit(false);
                    mReaderActivity.connectBtnEdit(true);
                    mReaderActivity.initUI();
                    mReaderActivity.reader = null;
                    mReaderActivity.regionListEdit(true);
                    System.gc();
                }
            } else {
                LoggerUtil.debug(TAG, "OperationStatus is true");

                mReaderActivity.initUI();

                if (doConnect) {
                    mReaderActivity.setConnectBtnText("Disconnect");
                    mReaderActivity.connectBtnEdit(true);
                    mReaderActivity.readBtnEdit(true);
                    mReaderActivity.tagOpBtnEdit(true);
                    mReaderActivity.regionListEdit(false);
                    mReaderActivity.baudrateListEdit(false);

                    mReaderActivity.updateSettings();
                } else {
                    mReaderActivity.setConnectBtnText("Connect");
                    mReaderActivity.showLayoutFirmware(false);
                    mReaderActivity.showFirmwareHint(true);
                    mReaderActivity.readBtnEdit(false);
                    mReaderActivity.tagOpBtnEdit(false);
                    if (!mReaderActivity.enableBarcode.isChecked()) {
                        mReaderActivity.connectBtnEdit(true);
                    }
                    mReaderActivity.enableBarcode();
                    mReaderActivity.reader = null;
                    mReaderActivity.regionListEdit(true);
                    mReaderActivity.baudrateListEdit(true);
                    System.gc();

                    this.cancel(false);
                }
            }
        }
    }
}
