package com.unitech.rfidreader.Listener;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.LinearLayout;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

import com.thingmagic.ReadExceptionListener;
import com.thingmagic.ReadListener;
import com.thingmagic.Reader;
import com.thingmagic.ReaderException;
import com.thingmagic.SerialReader;
import com.thingmagic.StatusListener;
import com.thingmagic.TagReadData;
//import com.unitech.rfidreader.BuildConfig;
import com.unitech.rfidreader.R;
import com.unitech.rfidreader.ReaderActivity;
import com.unitech.rfidreader.TagRecord;
import com.unitech.util.Beeper;
import com.unitech.util.LoggerUtil;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;

@SuppressLint("StaticFieldLeak")
public class ServiceListener implements View.OnClickListener {
    private final String TAG = "ServiceListener";

    private int redColor = 0xffff0000;
    private int textColor = 0xff000000;

    private TableLayout table;
    private LayoutInflater inflater;
    private ArrayList<String> addedEPCRecords = new ArrayList<>();

    private ConcurrentHashMap<String, TagRecord> epcToReadDataMap = new ConcurrentHashMap<>();
    private int uniqueRecordCount = 0;
    private int totalTagCount = 0;
    private long queryStartTime = 0;
    private long queryStopTime = 0;

    private ReaderActivity mReaderActivity;
    private Timer timer = new Timer();

    private int crcCount = 0;

    private ReentrantLock readerLock;
    private ReentrantLock stopLock, switchLock;
    private Condition stopCondition;
    private ExecutorService executor;

    private ReadThread readThread;
    private Beeper beeper;

//    private SectionsStateAdapter sectionsStateAdapter = null;
//    private TabLayout tabLayout = null;
//    private ViewPager2 viewPager = null;

    public ServiceListener(ReaderActivity readerActivity) {
        mReaderActivity = readerActivity;
        findAllViewsById();
        beeper = Beeper.getInstance();

        readerLock = new ReentrantLock();
        switchLock = new ReentrantLock();

        stopLock = new ReentrantLock();
        stopCondition = stopLock.newCondition();

        executor = Executors.newSingleThreadExecutor();
    }

    private void findAllViewsById() {
        inflater = (LayoutInflater) mReaderActivity.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
//        errorMsgView = mReaderActivity.findViewById(R.id.error_messages_view);
        textColor = mReaderActivity.getSearchResultTextColor();
        table = (TableLayout) mReaderActivity.findViewById(R.id.tablelayout);
    }

    private void readButtonStatus(boolean setRead, int enable) {
        mReaderActivity.sendBroadcast(new Intent("ACTION_LOCATE_READING")
                .putExtra("start", setRead)
                .putExtra("enable", enable)
        );

        if (setRead) {
            mReaderActivity.setReadBtnText(mReaderActivity.getString(R.string.stop_reading));
        } else {
            mReaderActivity.setReadBtnText(mReaderActivity.getString(R.string.start_reading));
        }

        switch (enable) {
            case 0:
                mReaderActivity.readBtnEdit(false);
                break;
            case 1:
                mReaderActivity.readBtnEdit(true);
                break;
        }
    }

    private void tagOptionButtonStatus(boolean enable) {
        mReaderActivity.tagOpBtnEdit(enable);
    }

    @Override
    public void onClick(View arg0) {
        try {
            if (mReaderActivity.getReadBtnText().equals(mReaderActivity.getResources().getString(R.string.stop_reading))) {
                stopReading();
            } else if (mReaderActivity.getReadBtnText().equals(mReaderActivity.getResources().getString(R.string.start_reading))) {
                startReading();
            }
        } catch (Exception ex) {
            LoggerUtil.error(TAG, "Exception", ex);
        }
    }

    public void startReading() {
        if (readThread != null) {
            if (!readThread.isStandy()) {
                LoggerUtil.debug(TAG, readThread.isReading() ? "It is reading" : "It is stoping");
                return;
            }
        }

        LoggerUtil.debug(TAG, "Trigger start reading");
        switchReading(true);
    }

    public void stopReading() {
        if (readThread != null) {
            if (!readThread.isReading())
                return;
        }
        switchReading(false);
    }

    private void switchReading(boolean enable) {
        if (!switchLock.tryLock()) {
            LoggerUtil.debug(TAG, "Switch lock is been locked");
            return;
        }
        if (enable) {
            if (readThread != null && !readThread.isStandy()) {
                LoggerUtil.warn(TAG, "Read thread is still working.");
                return;
            }
//            mReaderActivity.setReadBtnText(mReaderActivity.getResources().getString(R.string.stop_reading));
            readButtonStatus(true, -1);

            clearTagRecords();

            readThread = new ReadThread(mReaderActivity, "asyncRead");
            readThread.execute();
        } else if (!enable && !stopLock.isLocked()) {

            if (isReading())
                executor.submit(runnableStopReading);
        }
        switchLock.unlock();
    }

    private Runnable runnableStopReading = new Runnable() {
        @Override
        public void run() {
            if (isReading() && stopLock.tryLock()) {
                try {
                    LoggerUtil.debug(TAG, "Trigger stop reading");
                    readThread.setReading(false);
                    mReaderActivity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            readButtonStatus(true, 0);
                            mReaderActivity.setReadBtnText(mReaderActivity.getResources().getString(R.string.stopping));
                        }
                    });
                    stopCondition.await(5, TimeUnit.SECONDS);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                stopLock.unlock();
            }
        }
    };

    public synchronized boolean isReading() {
        if (readThread != null) {
            return readThread.isReading();
        } else {
            return false;
        }
    }

    public synchronized boolean isStandy() {
        if (readThread != null) {
            return readThread.isStandy();
        } else {
            return true;
        }
    }
    //---------------------------------------------------------------------------------------------

    public void clearTagRecords() {
        addedEPCRecords.clear();
        epcToReadDataMap.clear();
        table.removeAllViews();
        mReaderActivity.setSearchResultCountText("");
        mReaderActivity.setTotalTagCountText("");
        mReaderActivity.setAvgReadText("");
        mReaderActivity.setReadingTimeText("");
        mReaderActivity.setTemperatureText("");
        uniqueRecordCount = 0;
        totalTagCount = 0;
        queryStartTime = System.currentTimeMillis();
    }

    @SuppressLint("StaticFieldLeak")
    public class ReadThread extends AsyncTask<Void, Integer, ConcurrentHashMap<String, TagRecord>> {
        private final String TAG = "ReadThread";
        private boolean reading = true, running = false, standy = true;
//        private ReaderActivity mReaderActivity;

        private final DecimalFormat df = new DecimalFormat("#.##");

        private String temp_str = null, avg_str = null;
        private String operation;
        private boolean exceptionOccur = false;
        private List<String> exception;

        private Reader mReader;
        private int Temperature = 0;

        private int readLoop = 0;

        ReadExceptionListener exceptionListener = new TagReadExceptionReceiver();
        ReadListener readListener = new PrintListener();
        StatusListener statusListener = new ReaderStatusListener();

        ReadThread(ReaderActivity activity, String operation) {
            this.operation = operation;
//            mReaderActivity = activity;
            mReader = mReaderActivity.reader;
            exception = new ArrayList<>();
        }

        @Override
        protected void onPreExecute() {
            setRunning(true);
            clearTagRecords();
            exception.clear();

            mReaderActivity.connectBtnEdit(false);

//            readButtonStatus(false, 0);
            mReaderActivity.setReadBtnText(mReaderActivity.getResources().getString(R.string.starting));

            mReaderActivity.disableBarcode();
            mReaderActivity.tagOpBtnEdit(false);
            mReaderActivity.beepCheckBoxEdit(false);

            mReaderActivity.setSearchResultTextColor(textColor);
            mReaderActivity.setSearchResultCountText(mReaderActivity.getResources().getString(R.string.reading_tags));

            addedEPCRecords = new ArrayList<>();
            epcToReadDataMap = new ConcurrentHashMap<>();
            exceptionOccur = false;
        }

        @Override
        protected ConcurrentHashMap<String, TagRecord> doInBackground(Void... params) {
            try {
                readerLock.lock();
                setReading(true);
                mReaderActivity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        readButtonStatus(true, 1);
                    }
                });

                mReader.addReadExceptionListener(exceptionListener);
                mReader.addReadListener(readListener);
                mReader.addStatusListener(statusListener);

                mReader.startReading();

                readerLock.unlock();

                queryStartTime = System.currentTimeMillis();
                queryStopTime = queryStartTime;
                refreshReadRate();
                while (isReading()) {
                    /* Waiting till stop reading button is pressed */
                    Thread.sleep(5);
                }
                LoggerUtil.debug(TAG, "Reading end...");
            } catch (Exception ex) {
                exception.add(ex.getMessage());
                exceptionOccur = true;
                LoggerUtil.error(TAG, "Exception while reading :", ex);
            } finally {
                LoggerUtil.debug(TAG, "ReadThread stopping");
                timer.cancel();
                mReader.removeReadListener(readListener);
                mReader.removeReadExceptionListener(exceptionListener);
                mReader.removeStatusListener(statusListener);
                if (!exceptionOccur) {
                    mReader.stopReading();
                }
                standy = true;
                queryStopTime = System.currentTimeMillis();
                LoggerUtil.debug(TAG, "ReadThread stopped");
            }

            return epcToReadDataMap;
        }

        class PrintListener implements ReadListener {
            public void tagRead(Reader r, final TagReadData tr) {
                parseTag(tr);
//                readThread.parseTag(tr);
            }
        }

        class ReaderStatusListener implements StatusListener {
            @Override
            public void statusMessage(Reader reader, SerialReader.StatusReport[] statusReports) {
                if (reader == mReader) {
                    for (int i = 0; i < statusReports.length; i++) {
                        if (statusReports[i] instanceof SerialReader.TemperatureStatusReport) {
                            LoggerUtil.debug(TAG, statusReports[i].toString());
                            Temperature = ((SerialReader.TemperatureStatusReport) statusReports[i]).getTemperature();

                            if (Temperature > 70) {
                                final String ex = "The module temperature rises above 70°C and App stops reading to protect module. ";
                                LoggerUtil.warn(TAG, String.format("Module exception : %s", ex));
//                                readThread.setReading(false);
                                setReading(false);
                                mReaderActivity.runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        AlertDialog deleteAlert = new AlertDialog.Builder(mReaderActivity).create();
                                        deleteAlert.setTitle("Module Overheat");
                                        deleteAlert.setMessage(ex);
                                        deleteAlert.show();
                                    }
                                });
                            }

                        } else if (statusReports[i] instanceof SerialReader.FrequencyStatusReport) {
                            LoggerUtil.debug(TAG, statusReports[i].toString());
                        }
                    }
                }
            }
        }

        class TagReadExceptionReceiver implements ReadExceptionListener {
            public void tagReadException(Reader r, ReaderException re) {
                if (re.getMessage().contains("The module has detected high return loss")
                        || re.getMessage().contains("Tag ID buffer full")
                        || re.getMessage().contains("No tags found")) {
                    // exception = "No connected antennas found";
                    /* Continue reading */
                    LoggerUtil.warn(TAG, "Waring exception: ", re);
                } else {
                    LoggerUtil.error(TAG, "Reader exception : ", re);
                    exception.add(re.getMessage());
                    exceptionOccur = true;
                    ///readThread.setReading(false);
                    setReading(false);
                    readThread.publishProgress(-1);
                }
            }
        }

        private void refreshReadRate() {
            timer = new Timer();
            timer.schedule(new TimerTask() {
                @Override
                public void run() {
                    publishProgress(0);
                }
            }, 100, 100);
        }

        private void parseTag(TagReadData tr) {
            totalTagCount += tr.getReadCount();
            String epcString = tr.getTag().epcString();
//            Log.d(TAG, String.format("parseTag: %d read count, epc='%s'", tr.getReadCount(), tr.getTag().epcString()));
            if (mReaderActivity.mActionView.getVisibility() != View.VISIBLE) {
                if (tr.getReadCount() > 0) {
                    beeper.beep();
                }

                if (epcToReadDataMap.keySet().contains(epcString)) {
                    TagRecord tempTR = epcToReadDataMap.get(epcString);
                    tempTR.setRssi(tr.getRssi());
                    tempTR.readCount += tr.getReadCount();
                } else {
                    TagRecord tagRecord = new TagRecord();
                    tagRecord.setEpcString(epcString);
                    tagRecord.setReadCount(tr.getReadCount());
                    tagRecord.setRssi(tr.getRssi());
                    epcToReadDataMap.put(epcString, tagRecord);
                }
            } else {

                if (mReaderActivity.ActionTabPosition == 4) {
                    mReaderActivity.sendBroadcast(new Intent("ACTION_LOCATE_DATA")
                            .putExtra("LOCATE_EPC", epcString)
                            .putExtra("LOCATE_RSSI", tr.getRssi()));
                }
            }


        }

        @Override
        protected void onProgressUpdate(Integer... progress) {
            int progressToken = progress[0];
            if (progressToken == -1) {
                mReaderActivity.setErrorMsgTextColor(redColor);
                StringBuilder sb = new StringBuilder("ERROR :");
                for (String exMsg : exception) {
                    sb.append("\n");
                    sb.append(exMsg);
                }
                temp_str = sb.toString();
                mReaderActivity.showErrorMsg(true);
                mReaderActivity.setErrorMsg(temp_str);

                if (++readLoop >= 100 || !isReading()) {
                    readLoop = 0;
                }
            } else {
                populateSearchResult(epcToReadDataMap);
                if (!exceptionOccur) {
                    mReaderActivity.setSearchResultTextColor(textColor);
                    if (crcCount == 0) {
                        mReaderActivity.setSearchResultCountText(Html.fromHtml("<b>Unique Tags :</b> " + epcToReadDataMap.keySet().size()));
                    } else {
                        mReaderActivity.setSearchResultCountText(Html.fromHtml("<b>CRC : " + crcCount + " Unique Tags :</b> " + epcToReadDataMap.keySet().size()));
                    }
                    mReaderActivity.setTotalTagCountText(Html.fromHtml("<b>Total Tags  :</b> " + totalTagCount));

                    long elapsedTime = System.currentTimeMillis() - queryStartTime;
                    double timeTaken = ((double) elapsedTime / 1000);
                    temp_str = df.format(timeTaken) + " sec";

                    mReaderActivity.setReadingTimeText(Html.fromHtml("<b>Time:</b> " + temp_str));
                    avg_str = df.format(totalTagCount / timeTaken);
                    mReaderActivity.setAvgReadText(Html.fromHtml("<b>Avg. Reads Rate:</b> " + avg_str));

                    // Update temperature
                    mReaderActivity.setTemperatureText(Html.fromHtml("<b>Temperature:</b> " + Temperature));

                    if (++readLoop >= 100) {
                        LoggerUtil.info(TAG, String.format("Unique Tags: %d, Total Tags: %d, Time: %s, Avg. Reads Rate: %s",
                                epcToReadDataMap.keySet().size(),
                                totalTagCount,
                                temp_str,
                                avg_str
                        ));
                        readLoop = 0;
                    }
                }
            }
        }

        @Override
        protected void onPostExecute(ConcurrentHashMap<String, TagRecord> epcToReadDataMap) {
            timer.cancel();
            String temp_str;
            if (exceptionOccur) {
                crcCount++;
                mReaderActivity.setErrorMsgTextColor(redColor);
                StringBuilder sb = new StringBuilder("ERROR :");
                for (String exMsg : exception) {
                    sb.append("\n");
                    sb.append(exMsg);
                }
                temp_str = sb.toString();
                mReaderActivity.setErrorMsg(temp_str);
                mReaderActivity.setTotalTagCountText("");
                if (totalTagCount > 0 && !operation.equalsIgnoreCase("syncRead")) {
                    mReaderActivity.setTotalTagCountText(Html.fromHtml("<b>Total Tags  :</b> " + totalTagCount));
                }
            } else {
                if (crcCount == 0) {
                    mReaderActivity.setSearchResultCountText(Html.fromHtml("<b>Unique Tags :</b> " + epcToReadDataMap.keySet().size()));
                } else {
                    mReaderActivity.setSearchResultCountText(Html.fromHtml("<b>CRC : " + crcCount + " Unique Tags :</b> " + epcToReadDataMap.keySet().size()));
                }

                mReaderActivity.setTotalTagCountText(Html.fromHtml("<b>Total Tags  :</b> " + totalTagCount));
                populateSearchResult(epcToReadDataMap);

                long elapsedTime = queryStopTime - queryStartTime;
                double timeTaken = ((double) elapsedTime / 1000);
                temp_str = df.format(timeTaken) + " sec";
                mReaderActivity.setReadingTimeText(Html.fromHtml("<b>Time:</b> " + temp_str));
                mReaderActivity.setAvgReadText(Html.fromHtml("<b>Avg. Reads Rate:</b> " + df.format(totalTagCount / timeTaken)));

                LoggerUtil.info(TAG, String.format("Unique Tags: %d, Total Tags: %d, Time: %s, Avg. Reads Rate: %s",
                        epcToReadDataMap.keySet().size(),
                        totalTagCount,
                        temp_str,
                        df.format(totalTagCount / timeTaken)
                ));
            }

            if (exceptionOccur && !exception.isEmpty() && !exception.get(exception.size() - 1).equalsIgnoreCase("No connected antennas found")) {
                disconnectReader();
                mReaderActivity.enableBarcode();
                mReaderActivity.beepCheckBoxEdit(true);
            } else {
                beeper.stopAllBeep();
                mReaderActivity.enableBarcode();
                mReaderActivity.beepCheckBoxEdit(true);

                readButtonStatus(false, 1);
                tagOptionButtonStatus(true);

                mReaderActivity.connectBtnEdit(true);
            }
            setRunning(false);
            stopLock.lock();
            stopCondition.signalAll();
            stopLock.unlock();
        }

        private void disconnectReader() {
            mReaderActivity.setConnectBtnText("Connect");
            mReaderActivity.connectBtnEdit(true);

            readButtonStatus(false, 0);
            tagOptionButtonStatus(false);

            mReaderActivity.beepCheckBoxEdit(true);

            mReaderActivity.showLayoutFirmware(false);
            mReaderActivity.showFirmwareHint(true);

            mReaderActivity.reader.destroy();
            mReaderActivity.reader = null;

            mReaderActivity.RFIDModulePowerCtrl(false);
            mReaderActivity.RFIDModulePowerCtrl(true);

            if (!exceptionOccur) {
                mReaderActivity.setSearchResultCountText("");
                mReaderActivity.setAvgReadText("");
                mReaderActivity.setReadingTimeText("");
                mReaderActivity.setTemperatureText("");
            }
            beeper.stopAllBeep();
        }

        @SuppressLint("InflateParams")
        private void populateSearchResult(ConcurrentHashMap<String, TagRecord> epcToReadDataMap) {
            try {
                Set<String> keySet = epcToReadDataMap.keySet();

                TableRow fullRow = null;
                TextView nr = null;
                TextView epcValue = null;
                TextView rssiView = null;
                TextView dataView = null;
                TextView countView = null;
                boolean isEmbeddedRead = false;

                for (String epcString : keySet) {
                    final TagRecord tagRecordData = epcToReadDataMap.get(epcString);

                    if (!addedEPCRecords.contains(epcString)) {
                        addedEPCRecords.add(epcString);
                        uniqueRecordCount = addedEPCRecords.size();
                        if (inflater != null) {
                            fullRow = (TableRow) inflater.inflate(R.layout.row, null, true);
                            fullRow.setClickable(true);
                            fullRow.setId(uniqueRecordCount);

                            if (fullRow != null) {
                                nr = (TextView) fullRow.findViewById(R.id.nr);

                                if (nr != null) {
                                    nr.setText(String.valueOf(uniqueRecordCount));
                                    nr.setWidth(mReaderActivity.rowNumberWidth);
                                    epcValue = (TextView) fullRow.findViewById(R.id.EPC);

                                    if (epcValue != null) {
                                        epcValue.setOnClickListener(new OnClickListener() {
                                            @Override
                                            public void onClick(View v) {

//                                                mReaderActivity.setTagEPCText(tagRecordData.getEpcString());

                                                if (mReaderActivity.getReadBtnText().equals(mReaderActivity.getString(R.string.start_reading)) &&
                                                        mReaderActivity.reader != null &&
                                                        mReaderActivity.getConnectBtnText().equals(mReaderActivity.getString(R.string.Disconnect)) &&
                                                        mReaderActivity.serviceListener != null
                                                ) {
                                                    mReaderActivity.targetEPC = tagRecordData.getEpcString();
                                                    mReaderActivity.sectionsStateAdapter.SetEPCString(tagRecordData.getEpcString());
                                                    mReaderActivity.sectionsStateAdapter.notifyDataSetChanged();
                                                    mReaderActivity.showSettingsOrTagsView(2);
                                                }
                                            }
                                        });
                                        epcValue.setText(tagRecordData.getEpcString());
                                        epcValue.setWidth(mReaderActivity.epcDataWidth);
                                        rssiView = (TextView) fullRow.findViewById(R.id.RSSI);
                                        if (rssiView != null) {
                                            rssiView.setText(String.valueOf(tagRecordData.getRssi()));
                                            rssiView.setWidth(mReaderActivity.epcRSSIWidth);
                                        }
                                        countView = (TextView) fullRow.findViewById(R.id.COUNT);
                                        if (countView != null) {
                                            countView.setText(String.valueOf(tagRecordData.getReadCount()));
                                            countView.setWidth(mReaderActivity.epcCountWidth);
                                        }
                                        if (isEmbeddedRead) {
                                            dataView = (TextView) fullRow.findViewById(R.id.DATA);
                                            if (dataView != null) {
                                                dataView.setVisibility(View.VISIBLE);
                                                dataView.setText(Arrays.toString(tagRecordData.getData()));
                                            }
                                        }
                                        table.addView(fullRow);
                                    }
                                }
                            }
                        }
                    } else {
                        fullRow = (TableRow) table.getChildAt(addedEPCRecords.indexOf(epcString));
                        if (fullRow != null) {
                            rssiView = (TextView) fullRow.getChildAt(2);
//                            if (rssiView != null && Integer.valueOf(rssiView.getText().toString()) != tagRecordData.getRssi()) {
//                                rssiView.setText(String.valueOf(tagRecordData.getRssi()));
//                            }

                            if (rssiView != null && tagRecordData != null) {
                                int rssi = tagRecordData.getRssi();
                                if (Integer.parseInt(rssiView.getText().toString()) != rssi) {
                                    rssiView.setText(String.valueOf(tagRecordData.getRssi()));
                                }
                            }

                            countView = (TextView) fullRow.getChildAt(4);
                            if (countView != null && Integer.valueOf(countView.getText().toString()) != tagRecordData.getReadCount()) {
                                countView.setText(String.valueOf(tagRecordData.getReadCount()));
                            }
                        }
                    }
                }
            } catch (Exception ex) {
                LoggerUtil.error(TAG, "Exception while populating tags :", ex);
            }
        }

        public boolean isReading() {
            return reading;
        }

        public boolean isStandy() {
            return standy;
        }

        private void setReading(boolean reading) {
            if (reading)
                standy = false;

            this.reading = reading;

            mReaderActivity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    //myTabLayoutMediator.isRunning(reading);
                    mReaderActivity.viewPager.setUserInputEnabled(!mReaderActivity.serviceListener.isReading());
                    LinearLayout tabStrip = ((LinearLayout) mReaderActivity.tabLayout.getChildAt(0));
                    if (mReaderActivity.serviceListener.isReading()) {
                        tabStrip.setEnabled(false);
                        for (int i = 0; i < tabStrip.getChildCount(); i++) {
                            tabStrip.getChildAt(i).setClickable(false);
                        }
                    } else {
                        tabStrip.setEnabled(true);
                        for (int i = 0; i < tabStrip.getChildCount(); i++) {
                            tabStrip.getChildAt(i).setClickable(true);
                        }
                    }

                }
            });
        }

        public boolean isRunning() {
            return running;
        }

        private synchronized void setRunning(boolean r) {
            running = r;
        }
//
//        public static void setTemperature(int temp) {
//            Temperature = temp;
//        }
    }
}