package com.unitech.util;

import android.content.Context;
import android.content.SharedPreferences;

public class RFIDConfig {

    private final String Custom_Conf = "Custom";

    private final String Key_Operation_Mode = "OperationMode";
    private final String Key_Reader_Timeout = "ReaderTimeout";
    private final String Key_Antenna_On_Time = "OnTime";
    private final String Key_Antenna_Off_Time = "OffTime";
    private final String Key_Session = "Session";
    private final String Key_Target = "Target";
    private final String Key_Fast_Search = "FastSearch";
    private final String Key_Q_Value = "QValue";
    private final String Key_Encoding = "Encoding";
    private final String Key_TARI = "TARI";
    private final String Key_BLF = "BLF";
    private final String Key_Power = "Power";

    private SharedPreferences properties;

    private static Context appContext;

    public RFIDConfig(Context context) {
        appContext = context;
        loadConf();
    }

    /**
     * The default is Cycle Count
     */
    private void setDefault() {

        setOperationMode(1);

        setReaderTimeout(1000);

        setAntennaOnTime(300);

        setAntennaOffTime(300);

        setPower(20);

        setBLF(0);

        setTARI(0);

        setSession(1);

        setEncoding(3);

        setQValue(0);

        setTarget(2);

        setFastSearch(false);
    }

    public void loadConf() {

        properties = appContext.getApplicationContext().getSharedPreferences(Custom_Conf, Context.MODE_PRIVATE);

        //If there is no settings, use the default
        if (!properties.contains(Key_Operation_Mode)) {
            setDefault();
            return;
        }

        if (!properties.contains(Key_Reader_Timeout))
            setReaderTimeout(1000);

        if (!properties.contains(Key_Antenna_On_Time))
            setAntennaOnTime(300);

        if (!properties.contains(Key_Antenna_Off_Time))
            setAntennaOffTime(300);

        if (!properties.contains(Key_Power))
            setPower(20);

        if (!properties.contains(Key_BLF))
            setBLF(0);

        if (!properties.contains(Key_TARI))
            setTARI(0);

        if (!properties.contains(Key_Session))
            setSession(1);

        if (!properties.contains(Key_Encoding))
            setEncoding(3);

        if (!properties.contains(Key_Q_Value))
            setQValue(0);

        if (!properties.contains(Key_Target))
            setTarget(2);

        if (!properties.contains(Key_Fast_Search))
            setFastSearch(false);
    }

    public void setOperationMode(int mode) {
        properties.edit().putInt(Key_Operation_Mode, mode).commit();
    }

    public int getOperationMode() {
        if (!properties.contains(Key_Operation_Mode))
            setOperationMode(1);

        return properties.getInt(Key_Operation_Mode, 1);
    }

    //region Reader Timeout
    public void setReaderTimeout(int timeout) {
        properties.edit().putInt(Key_Reader_Timeout, timeout).commit();
    }

    /**
     * Get the Reader Timeout value
     *
     * @return The value under current Operation Mode
     */
    public int getReaderTimeout() {
        return getReaderTimeout(getOperationMode());
    }

    /**
     * Get the Reader Timeout value
     *
     * @param mode Under which Operation Mode to read
     * @return Return the value under specific Operation Mode
     */
    public int getReaderTimeout(int mode) {
        if (!properties.contains(Key_Reader_Timeout))
            setReaderTimeout(1000);

        if (mode != 2)
            return 1000;

        return properties.getInt(Key_Reader_Timeout, 1000);
    }
    //endregion

    //region Antenna On Time
    public void setAntennaOnTime(int onTime) {
        properties.edit().putInt(Key_Antenna_On_Time, onTime).commit();
    }

    public int getAntennaOnTime() {
        return getAntennaOnTime(getOperationMode());
    }

    public int getAntennaOnTime(int mode) {
        if (!properties.contains(Key_Antenna_On_Time))
            setAntennaOnTime(300);

        if (mode == 0)
            return 600;
        else if (mode == 1)
            return 300;

        return properties.getInt(Key_Antenna_On_Time, 300);
    }
    //endregion

    //region Antenna Off Time
    public void setAntennaOffTime(int offTime) {
        properties.edit().putInt(Key_Antenna_Off_Time, offTime).commit();
    }

    public int getAntennaOffTime() {
        return getAntennaOffTime(getOperationMode());
    }

    public int getAntennaOffTime(int mode) {
        if (!properties.contains(Key_Antenna_Off_Time))
            setAntennaOffTime(300);

        if (mode == 0)
            return 0;
        else if (mode == 1)
            return 300;

        return properties.getInt(Key_Antenna_Off_Time, 300);
    }
    //endregion

    //region Power
    public void setPower(int power) {
        properties.edit().putInt(Key_Power, power).commit();
    }

    public int getPower() {
        return getPower(getOperationMode());
    }

    public int getPower(int mode) {
        if (!properties.contains(Key_Power))
            setPower(20);

        if (mode == 0)
            return 16;
        else if (mode == 1)
            return 20;

        return properties.getInt(Key_Power, 20);
    }
    //endregion

    //region Session
    public void setSession(int session) {
        properties.edit().putInt(Key_Session, session).commit();
    }

    public int getSession() {
        return getSession(getOperationMode());
    }

    public int getSession(int mode) {
        if (!properties.contains(Key_Session))
            setSession(1);

        if (mode == 0)
            return 0;
        else if (mode == 1)
            return 1;

        return properties.getInt(Key_Session, 1);
    }
    //endregion

    //region Target
    public void setTarget(int target) {
        properties.edit().putInt(Key_Target, target).commit();
    }

    public int getTarget() {
        return getTarget(getOperationMode());
    }

    public int getTarget(int mode) {
        if (!properties.contains(Key_Target))
            setTarget(2);

        if (mode != 2)
            return 2;

        return properties.getInt(Key_Target, 2);
    }
    //endregion

    //region BLF
    public void setBLF(int blf) {
        properties.edit().putInt(Key_BLF, blf).commit();
    }

    public int getBLF() {
        return getBLF(getOperationMode());
    }

    public int getBLF(int mode) {
        if (!properties.contains(Key_BLF))
            setBLF(0);

        if (mode == 0)
            return 1;
        else if (mode == 1)
            return 0;

        return properties.getInt(Key_BLF, 0);
    }
    //endregion BLF

    //region TARI
    public void setTARI(int tari) {
        properties.edit().putInt(Key_TARI, tari).commit();
    }

    public int getTARI() {
        return getTARI(getOperationMode());
    }

    public int getTARI(int mode) {
        if (!properties.contains(Key_TARI))
            setTARI(0);

        if (mode == 0)
            return 2;
        else if (mode == 1)
            return 0;

        return properties.getInt(Key_TARI, 0);
    }
    //endregion

    //region Encoding
    public void setEncoding(int encoding) {
        properties.edit().putInt(Key_Encoding, encoding).commit();
    }

    public int getEncoding() {
        return getEncoding(getOperationMode());
    }

    public int getEncoding(int mode) {
        if (!properties.contains(Key_Encoding))
            setEncoding(3);

        if (mode == 0)
            return 0;
        else if (mode == 1)
            return 3;

        return properties.getInt(Key_Encoding, 3);
    }
    //endregion

    //region Q
    public void setQValue(int q) {
        properties.edit().putInt(Key_Q_Value, q).commit();
    }

    public int getQValue() {
        return getQValue(getOperationMode());
    }

    public int getQValue(int mode) {
        if (!properties.contains(Key_Q_Value))
            setQValue(0);

        if (mode != 2)
            return 0;

        return properties.getInt(Key_Q_Value, 0);
    }
    //endregion

    //region Fast Search
    public void setFastSearch(boolean fastSearch) {
        properties.edit().putBoolean(Key_Fast_Search, fastSearch).commit();
    }

    public boolean getFastSearch() {
        return getFastSearch(getOperationMode());
    }

    public boolean getFastSearch(int mode) {
        if (!properties.contains(Key_Fast_Search))
            setFastSearch(false);

        if (mode != 2)
            return false;

        return properties.getBoolean(Key_Fast_Search, false);
    }
    //endregion
}
